#include "libscio.hpp"
#include <unistd.h>
#include <cstring>

sc::io::memstreambuf::memstreambuf(char* mem, size_t size) {
	init(mem, size);
}

void sc::io::memstreambuf::init(char* mem, size_t size) {
	_mem = mem;
	_size = size;
	setg(_mem, _mem, _mem + _size);
}

std::streambuf::int_type sc::io::memstreambuf::uflow() {
	if (gptr() == _mem + _size) return EOF;
	char c = *gptr();
	setg(_mem, gptr() + 1, _mem + _size);
	return c;
}

std::streamsize sc::io::memstreambuf::xsgetn(char* s, std::streamsize n) {
	std::streamsize left = _mem + _size - gptr();
	std::streamsize tot_read = std::min(left, n);
	if (tot_read == 0) return EOF;
	std::memcpy(s, gptr(), tot_read);
	return tot_read;
}

std::streambuf::int_type sc::io::memstreambuf::overflow(std::streambuf::int_type c) {
	// TODO
	// if (c != EOF) {
	// 	int res;
	// 	if ((res = write(_fd, &c, 1)) != 1) {
	// 		return EOF;
	// 	}
	// 	else {
	// 		std::cerr << "write result = " << res << ", errno = " << errno << '\n';
	// 	}
	// }
	// _ch = c;
	// setp(&_ch, &_ch + 1);
	return c;
}

std::streamsize sc::io::memstreambuf::xsputn(const char* s, std::streamsize num) {
	// TODO
// 	int res = write(_fd, s, num);
// 	if (res != num) {
// 		std::cerr << "write result = " << res << ", errno = " << errno << '\n';
// 		return EOF;
// 	}
	// return res;
	return num;
}

std::streambuf::pos_type sc::io::memstreambuf::seekoff(std::streambuf::off_type off,
												  std::ios_base::seekdir dir,
												  std::ios_base::openmode /*which*/) {
	switch (dir) {
		case std::ios_base::beg: setg(_mem, _mem + off, _mem + _size); break;
		case std::ios_base::end: setg(_mem, _mem + _size - off, _mem + _size); break;
		case std::ios_base::cur: setg(_mem, gptr() + off, _mem + _size); break;
		default: throw std::runtime_error("invalid seekdir");
	}
	return gptr() - _mem;
}

std::streambuf::pos_type sc::io::memstreambuf::seekpos(std::streambuf::pos_type pos,
												  std::ios_base::openmode /*which*/) {
	setg(_mem, _mem + pos, _mem + _size);
	return gptr() - _mem;
}

std::streambuf::int_type sc::io::memstreambuf::pbackfail(std::streambuf::int_type /*c*/) {
	setg(_mem, gptr() - 1, _mem + _size);
	if (gptr() < _mem) return EOF;
	return 0;
}
